
package com.bd.movie.serviceapi;

import java.util.Date;
import java.util.List;

import com.bd.framework.accessapi.CreateAccess;
import com.bd.framework.accessapi.FindByExampleAccess;
import com.bd.movie.accessapi.LibraryAccessFactory;
import com.bd.movie.accessapi.PersonAccessFactory;
import com.bd.movie.domain.Engagement;
import com.bd.movie.domain.Library;
import com.bd.movie.domain.LibraryNotFoundException;
import com.bd.movie.domain.Media;
import com.bd.movie.domain.MediaCharacter;
import com.bd.movie.domain.Movie;
import com.bd.movie.domain.Person;
import com.bd.movie.domain.PhysicalMedia;
import com.bd.movie.serviceapi.LibraryService;
import com.bd.framework.util.HibernateUtil;

import junit.framework.TestCase;

/**
 * JUnit test.
 * @generated
 */
public class LibraryServiceTest extends TestCase {
    
    /** @generated */
    private LibraryService libraryService;
    
    private int libraryId;
    private String libraryName;

    /** @generated */
    protected void setUp() {
        setUpLibraryService();
    }
    
    protected void setUpLibraryService() {
        libraryService = new com.bd.movie.serviceimpl.LibraryServiceImpl();
        
        // Initialize some test data
        
        libraryName = "Test " + System.currentTimeMillis();
        Library library = new Library();
        library.setName(libraryName);
        
        addMovie1(library);

        
        addMovie2(library);
        
        CreateAccess<Library> createAccess = LibraryAccessFactory.getInstance().createCreateAccess();
        createAccess.setEntity(library);
        createAccess.execute();
        
        HibernateUtil.closeSession();
        
        libraryId = library.getId();    
    }
    
    private void addMovie1(Library library) {
        PhysicalMedia physicalMedia = new PhysicalMedia();
        physicalMedia.setLibrary(library);
        library.getMedia().add(physicalMedia);
        physicalMedia.setLocation("abc123");
        physicalMedia.setStatus("A");
        
        Movie movie = new Movie();
        movie.getPhysicalMedia().add(physicalMedia);
        physicalMedia.getMedia().add(movie);
        movie.setTitle("Pippi Lngstrump i Sderhavet");
        movie.setPlayLength(17);
        movie.setUrlIMDB("abc");
        
        MediaCharacter character = new MediaCharacter();
        character.getExistsInMedia().add(movie);
        movie.getCharacters().add(character);
        character.setName("Pippi");
    }
    
    private void addMovie2(Library library) {
        PhysicalMedia physicalMedia = new PhysicalMedia();
        physicalMedia.setLibrary(library);
        library.getMedia().add(physicalMedia);
        physicalMedia.setLocation("abc456");
        physicalMedia.setStatus("A");
        
        Movie movie = new Movie();
        movie.getPhysicalMedia().add(physicalMedia);
        physicalMedia.getMedia().add(movie);
        movie.setTitle("Die Another Day");
        movie.setPlayLength(18);
        movie.setUrlIMDB("abc");
        
        MediaCharacter character = new MediaCharacter();
        character.getExistsInMedia().add(movie);
        movie.getCharacters().add(character);
        character.setName("James Bond");
        
        
        Person person = new Person();
        person.setName("Pierce Brosnan");
        person.setAge(40);
        person.setSsn("123456");
        // avoid creation of duplicate person instances in DB
        FindByExampleAccess<Person> query = PersonAccessFactory.getInstance().createFindByExampleAccess();
        query.setExample(person);
        query.execute();
        if (!query.getResult().isEmpty()) {
            // use the existing instance
            person = query.getResult().iterator().next();
        }
        character.getPlayedBy().add(person);
        person.getPlays().add(character);
        
        Engagement engagement = new Engagement();
        engagement.setPerson(person);
        person.getEngagements().add(engagement);
        engagement.setMedia(movie);
        engagement.setRole("Actor");
    }


    public void testFindLibraryByName() throws Exception {
        Library library = libraryService.findLibraryByName(libraryName);
        assertNotNull(library);
        assertEquals(libraryName, library.getName());
        assertTrue(library.getId() > 0);
        
        assertTrue(HibernateUtil.isClosed());
    }
    
    public void testFindLibraryByNameNotFound() throws Exception {
        Library library = libraryService.findLibraryByName("not a library");
        assertNull(library);

        assertTrue(HibernateUtil.isClosed());
    }
     
    public void testCreateLibrary() throws Exception {
        String name = "TestCreateLibrary " + new Date();
        Library library = new Library();
        library.setName(name);
        libraryService.createLibrary(library);
        
        assertTrue(HibernateUtil.isClosed());
        
        Library foundLibrary = libraryService.findLibraryByName(name);
        assertNotNull(foundLibrary);
    }
     
    public void testFindMediaByName() throws Exception {
        String title = "Pippi Lngstrump i Sderhavet";
        List<Media> movieList = libraryService.findMediaByName(libraryId, title);
        assertNotNull(movieList);
        assertEquals(1, movieList.size());
        assertEquals(Movie.class, movieList.get(0).getClass());
        Movie movie = (Movie) movieList.get(0);
        assertEquals(title, movie.getTitle());
    }
     
    public void testFindMediaByCharacter() throws Exception {
        String characterName = "James Bond";
        List<Media> movieList = libraryService.findMediaByCharacter(libraryId, characterName);
        assertNotNull(movieList);
        assertEquals(1, movieList.size());
        assertEquals(Movie.class, movieList.get(0).getClass());
        Movie movie = (Movie) movieList.get(0);
        assertEquals("Die Another Day", movie.getTitle());
    }
     
    public void testFindPersonByName() throws Exception {
        String name = "Pierce Brosnan";
        Person person = libraryService.findPersonByName(name);
        assertNotNull(person);
        assertEquals(name, person.getName());
        
        assertFalse(person.getPlays().isEmpty());
        MediaCharacter playsCharacter = person.getPlays().iterator().next();
        assertEquals("James Bond", playsCharacter.getName());
        assertEquals(1, playsCharacter.getExistsInMedia().size());
        Media movie = playsCharacter.getExistsInMedia().iterator().next();
        assertEquals("Die Another Day", movie.getTitle());
        
        assertFalse(person.getEngagements().isEmpty());
        Engagement engagement = person.getEngagements().iterator().next();
        assertEquals("Actor", engagement.getRole());
        assertEquals("Die Another Day", engagement.getMedia().getTitle());
    }
     
   
} 